(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms')) :
    typeof define === 'function' && define.amd ? define('@ks89/ngx-codemirror', ['exports', '@angular/core', '@angular/forms'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.ks89 = global.ks89 || {}, global.ks89['ngx-codemirror'] = {}), global.ng.core, global.ng.forms));
}(this, (function (exports, core, forms) { 'use strict';

    /**
     * CodeMirror component
     * Usage :
     * <codemirror [(ngModel)]="data" [config]="{...}"></codemirror>
     */
    var CodemirrorComponent = /** @class */ (function () {
        /**
         * Constructor
         */
        function CodemirrorComponent() {
            // tslint:disable-next-line:no-output-native
            this.change = new core.EventEmitter();
            // tslint:disable-next-line:no-output-native
            this.focus = new core.EventEmitter();
            // tslint:disable-next-line:no-output-native
            this.blur = new core.EventEmitter();
            this.cursorActivity = new core.EventEmitter();
            this.val = '';
            if (typeof window !== 'undefined') {
                this.codemirror = require('codemirror');
            }
        }
        Object.defineProperty(CodemirrorComponent.prototype, "value", {
            get: function () {
                return this.val;
            },
            set: function (v) {
                if (v !== this.val) {
                    this.val = v;
                    this.onChange(v);
                }
            },
            enumerable: false,
            configurable: true
        });
        /**
         * On component view init
         */
        CodemirrorComponent.prototype.ngAfterViewInit = function () {
            this.config = this.config || {};
            this.codemirrorInit(this.config);
        };
        /**
         * Initialize codemirror
         */
        CodemirrorComponent.prototype.codemirrorInit = function (config) {
            var _this = this;
            if (!this.host) {
                throw new Error('Internal library error - host must be defined');
            }
            this.instance = this.codemirror.fromTextArea(this.host.nativeElement, config);
            this.instance.setValue(this.val);
            this.instance.on('change', function () {
                _this.updateValue(_this.instance.getValue());
            });
            this.instance.on('focus', function (instance, event) {
                _this.focus.emit({ instance: instance, event: event });
            });
            this.instance.on('cursorActivity', function (instance) {
                _this.cursorActivity.emit({ instance: instance });
            });
            this.instance.on('blur', function (instance, event) {
                _this.blur.emit({ instance: instance, event: event });
            });
        };
        /**
         * Value update process
         */
        CodemirrorComponent.prototype.updateValue = function (value) {
            this.value = value;
            this.onTouched();
            this.change.emit(value);
        };
        /**
         * Implements ControlValueAccessor
         */
        CodemirrorComponent.prototype.writeValue = function (value) {
            this.val = value || '';
            if (this.instance) {
                this.instance.setValue(this.val);
            }
        };
        CodemirrorComponent.prototype.onChange = function (_) { };
        CodemirrorComponent.prototype.onTouched = function () { };
        CodemirrorComponent.prototype.registerOnChange = function (fn) {
            this.onChange = fn;
        };
        CodemirrorComponent.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        return CodemirrorComponent;
    }());
    CodemirrorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'codemirror',
                    providers: [
                        {
                            provide: forms.NG_VALUE_ACCESSOR,
                            useExisting: core.forwardRef(function () { return CodemirrorComponent; }),
                            multi: true
                        }
                    ],
                    template: "<textarea #host></textarea>"
                },] }
    ];
    CodemirrorComponent.ctorParameters = function () { return []; };
    CodemirrorComponent.propDecorators = {
        config: [{ type: core.Input }],
        change: [{ type: core.Output }],
        focus: [{ type: core.Output }],
        blur: [{ type: core.Output }],
        cursorActivity: [{ type: core.Output }],
        host: [{ type: core.ViewChild, args: ['host',] }],
        value: [{ type: core.Input }]
    };

    var CodemirrorModule = /** @class */ (function () {
        function CodemirrorModule() {
        }
        return CodemirrorModule;
    }());
    CodemirrorModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [CodemirrorComponent],
                    imports: [],
                    exports: [CodemirrorComponent]
                },] }
    ];

    /*
     * Public API Surface of ngx-codemirror
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.CodemirrorComponent = CodemirrorComponent;
    exports.CodemirrorModule = CodemirrorModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ks89-ngx-codemirror.umd.js.map
