import { EventEmitter, Component, forwardRef, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * CodeMirror component
 * Usage :
 * <codemirror [(ngModel)]="data" [config]="{...}"></codemirror>
 */
class CodemirrorComponent {
    /**
     * Constructor
     */
    constructor() {
        // tslint:disable-next-line:no-output-native
        this.change = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.focus = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.blur = new EventEmitter();
        this.cursorActivity = new EventEmitter();
        this.val = '';
        if (typeof window !== 'undefined') {
            this.codemirror = require('codemirror');
        }
    }
    get value() {
        return this.val;
    }
    set value(v) {
        if (v !== this.val) {
            this.val = v;
            this.onChange(v);
        }
    }
    /**
     * On component view init
     */
    ngAfterViewInit() {
        this.config = this.config || {};
        this.codemirrorInit(this.config);
    }
    /**
     * Initialize codemirror
     */
    codemirrorInit(config) {
        if (!this.host) {
            throw new Error('Internal library error - host must be defined');
        }
        this.instance = this.codemirror.fromTextArea(this.host.nativeElement, config);
        this.instance.setValue(this.val);
        this.instance.on('change', () => {
            this.updateValue(this.instance.getValue());
        });
        this.instance.on('focus', (instance, event) => {
            this.focus.emit({ instance, event });
        });
        this.instance.on('cursorActivity', (instance) => {
            this.cursorActivity.emit({ instance });
        });
        this.instance.on('blur', (instance, event) => {
            this.blur.emit({ instance, event });
        });
    }
    /**
     * Value update process
     */
    updateValue(value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    }
    /**
     * Implements ControlValueAccessor
     */
    writeValue(value) {
        this.val = value || '';
        if (this.instance) {
            this.instance.setValue(this.val);
        }
    }
    onChange(_) { }
    onTouched() { }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
}
CodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'codemirror',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => CodemirrorComponent),
                        multi: true
                    }
                ],
                template: `<textarea #host></textarea>`
            },] }
];
CodemirrorComponent.ctorParameters = () => [];
CodemirrorComponent.propDecorators = {
    config: [{ type: Input }],
    change: [{ type: Output }],
    focus: [{ type: Output }],
    blur: [{ type: Output }],
    cursorActivity: [{ type: Output }],
    host: [{ type: ViewChild, args: ['host',] }],
    value: [{ type: Input }]
};

class CodemirrorModule {
}
CodemirrorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CodemirrorComponent],
                imports: [],
                exports: [CodemirrorComponent]
            },] }
];

/*
 * Public API Surface of ngx-codemirror
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodemirrorComponent, CodemirrorModule };
//# sourceMappingURL=ks89-ngx-codemirror.js.map
